severity = 1
exclude = RequireTidyCode RequirePodSections ProhibitPostfixControls RequireRcsKeywords RequireExplicitPackage RequirePodLinksIncludeText
include = Moose::ProhibitMultipleWiths Moose::ProhibitNewMethod Moose::RequireCleanNamespace Moose::RequireMakeImmutable
color = 1
verbose = 9

[BuiltinFunctions::ProhibitStringyEval]
allow_includes = 1

[CodeLayout::ProhibitTrailingWhitespace]

[Documentation::PodSpelling]
stop_words = Zilla accessors MetaProvides Plugin ModVersion undef ProviderObject yml ini metadata mungers hashref plugins parseable YAML metaproviders Plugins ProvideRecord's xdg DAGOLDEN MetaNoIndex plugin

[Subroutines::ProhibitUnusedPrivateSubroutines]
private_name_regex = _(?!build_)\w

[TestingAndDebugging::RequireUseStrict]
equivalent_modules = Moose

[TestingAndDebugging::RequireUseWarnings]
equivalent_modules = Moose

